/*
 *  (C) COPYRIGHT International Business Machines Corp. 1995,1996
 *
 * Filename : 40
 * Author   : M. Flickner
 * Date     : 8/16/96 18:09:00
 * Version  : 1.8.1.6
 * Purpose  : 
 *
 */

#include "Qbic.h"
#include "QbAbsCls.hpp"
#include "QbCreate.hpp"
#include "QbImgDat.hpp"
#include "QbQueryS.hpp"

//we do not need client for extenders
#if !defined(DMB_OS2) && !defined(DMB_WNT) 
#include "QbQueryC.hpp"
#endif


//
// define the functions to create instances of the classes
//
extern QbFeatureClass *NewQbColorFeatureClass() ;
extern QbFeatureClass *NewQbTextFeatureClass() ;
extern QbFeatureClass *NewQbTextureFeatureClass() ;
extern QbFeatureClass *NewQbColorHistogramFeatureClass() ;
extern QbFeatureClass *NewQbDrawFeatureClass() ;
extern QbFeatureClass *NewQbThumbnailClass() ;
#ifdef NEW_CODE
extern QbFeatureClass *NewQbGHashFeatureClass() ;
#endif


DMBEXPCP( QbFeatureClass *) QbCreateFeatureClass (const char *className )
{
  if( ! strcmp( className, "QbTextFeatureClass") ) 
    return  NewQbTextFeatureClass() ;
  if( ! strcmp( className, "QbColorHistogramFeatureClass") ) 
    return  NewQbColorHistogramFeatureClass() ;
  if( ! strcmp( className, "QbDrawFeatureClass") ) 
    return  NewQbDrawFeatureClass() ;
  if( ! strcmp( className, "QbColorFeatureClass") ) 
    return  NewQbColorFeatureClass() ;
  if( ! strcmp( className, "QbTextureFeatureClass") ) 
    return  NewQbTextureFeatureClass() ;
  if( ! strcmp( className, "QbThumbnailClass") ) 
    return  NewQbThumbnailClass() ;
#ifdef NEW_CODE
  if( ! strcmp( className, "QbGHashFeatureClass") ) 
    return  NewQbGHashFeatureClass() ;
#endif  
  return NULL ; 
}

extern QbConnectClass *NewQbDbmConnectClass() ; 

DMBEXPCP( QbConnectClass *) QbCreateConnectClass(const char *className )
{
  if( ! strcmp( className, "QbDbmConnectClass") )
    return NewQbDbmConnectClass();
  return NULL ; 
}


DMBEXPCP( QbGenericDataClass *) QbCreateGenericDataClass(const char *className)
{
  if( ! strcmp( className, "QbGenericImageDataClass" ) )
    return new QbGenericImageDataClass ;
  return NULL ; 
}

QbParameterClass *QbCreateParameterClass(const char *className)
{
    return new QbParameterClass ;
}

//
// create instances of query class
//
QbQueryClass *QbCreateQueryClass( const char *className )
{
// we do not need client for extenders  
#if !defined(DMB_AIX) && !defined(DMB_OS2) && !defined(DMB_WNT) 
  if( ! strcmp( className, "QbQueryClientClass" ) )
    return new QbQueryClientClass ;                
#endif    
  if( ! strcmp( className, "QbQueryServerClass" ) )
    return new QbQueryServerClass ;
  return NULL ; 
}


static QbConnectClass *connection ;

//
//To get things started you need to connect to a database.  To do this
//you need to know the class name for the QbConnectClass, the dataset
//name, and the mode, "r" or "w", you wish to connect with.  This is
//done once per address space in this implementation.  More
//sophisticated implementations are possible.
//
//I don't want caller deleting, disconnecting or connecting with the
//return value and I can enforce that using const.  
//
DMBEXPCP(const QbConnectClass *) QbGetConnection(const char *modeArg,
                                      const char *dsNameArg,
                                      const char *connectClassNameArg)

{
  static const char *defaultConnectClassName = {"QbDbmConnectClass"} ;
  static const char *defaultDsName = {"QbicData"} ;
  int rc ; 

  const char *dsName, *mode, *connectClassName ; 
  
  if( connection ) return connection ; // only one connection per address space

  mode = getenv("QbicConnectMode") ; 
  if( modeArg ) mode = modeArg ; // used passed mode
  
  dsName  = getenv("QbicDatabaseName") ; // get environment variable 
  if( dsNameArg ) dsName = dsNameArg ; // used passed arg
  if( NULL == dsName ) dsName = defaultDsName ; // lastly default

  connectClassName = getenv("QbicConnectClass") ; // get env variable
  if( connectClassNameArg ) connectClassName = connectClassName ;
  if( NULL == connectClassName )  connectClassName = defaultConnectClassName ;
  
  connection = QbCreateConnectClass( connectClassName ) ;
  rc = connection->Connect( dsName, mode ) ;
  if( rc ) {
    delete connection ;
    connection = NULL ; 
  } 
  return connection ;
}

//
// Process to Delete the connection
//
DMBEXPCP(int ) QbDestroyConnection( const QbConnectClass *connectArg )
{
  if( connectArg == connection ) {
    delete connection ;
    connection = NULL ;
  } else return -1 ;
  return 0 ;
}
