/*
 *  (C) COPYRIGHT International Business Machines Corp. 1995
 *
 * Filename : 52
 * Author   : Myron Flickner 
 * Date     : 8/9/96 13:54:43
 * Version  : 1.6.1.5
 * Purpose  : Main program to start query server
 */

static char sccsid[] = "QbQrySrv.cpp, qbicapi, dmb_12 8/9/96" ; 

#include <string.h>
#include <stdlib.h>

#include "QbQueryS.hpp"
#include "QbQueryC.hpp"
#include "QbCreate.hpp"

#include "QbSock.h"

//
// function to test if string is all digits
//
static int IsDigits( const char *buffer )
{
  for( int i = 0; buffer[i]; i++ )
    if ( ! isdigit(buffer[i] )) return 0 ;  
  return 1 ;  
}

/* maximum connection pending the server can have */
#define MAXCONNECTPEND 25
//
//QbQueryServer - setups the socket which clients connect to and
//issue queries through, and executes the valid commands/queries from
//the clients
//
static int QbQueryServer(char *dataSetName, int portNumber)
{
  int i, j;
  int socketDescriptor;
  struct sockaddr_in sockAddr;
  int sockAddrLen;
  int socketConnection;
  enum { bufferLength = 32 } ;
  char buffer[bufferLength] ; // space to get/put data
  char *querySpec, *datumValue ; 
  const char okay[] = {"okay"} ;
  unsigned long querySpecSize, nbytes, datumSize ;
  QbQueryClass *query ;
  QbDatumClass datum ;
  const QbKeyDatabaseClass *results ; 
  
  const QbConnectClass *qbicDataConnection ;

  QbInitSocket();
  /* create the socket */
  if ((socketDescriptor = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
    fprintf(stderr,"QueryServer: Error - Unable to create socket.\n");
    perror("QueryServer Error");

    QbCleanSocket();
    return(-1);
  }
  /* setup socket to reuse address */
  i = 1;
  if (setsockopt(socketDescriptor, SOL_SOCKET, SO_REUSEADDR, (char *)&i,
                 sizeof(i)) < 0) {
    fprintf(stderr,"QueryServer: Error - Unable to set reuse address.\n");
    perror("QueryServer Error");

    QbCloseSocket(socketDescriptor);

    return(-1);
  }
  /* setup socket to be kept alive */
  if (setsockopt(socketDescriptor, SOL_SOCKET, SO_KEEPALIVE, (char *)&i,
                 sizeof(i)) < 0) {
    fprintf(stderr,"QueryServer: Error - Unable to set keep alive.\n");
    perror("QueryServer Error");

    QbCloseSocket(socketDescriptor);
    return(-1);
  }
  /* bind socket essentially to a port */
  sockAddr.sin_port = htons(portNumber);
  sockAddr.sin_family = AF_INET;
  sockAddr.sin_addr.s_addr = htonl(INADDR_ANY);
  if ((bind(socketDescriptor, (struct sockaddr *)&sockAddr, sizeof(sockAddr)))
      < 0) {
    fprintf(stderr,"QueryServer: Error - Unable to bind socket.\n");
    perror("QueryServer Error");

    QbCloseSocket(socketDescriptor);
    return(-1);
  }
  /* setup socket to listen for connections upto the maximum number pending */
  listen(socketDescriptor, MAXCONNECTPEND);
  
  /* make a data set connection */
  qbicDataConnection = QbGetConnection("r", dataSetName);
  
  /* loop forever unless exit command issued */
  for ( ; ; ) {
    /* block for client connections */
    sockAddrLen = sizeof(sockAddr);
    if ((socketConnection = accept(socketDescriptor,
                                   (struct sockaddr *) &sockAddr,
                                   &sockAddrLen)) < 0) {
      fprintf(stderr,
        "QueryServer: Error - Unable to establish a connection to socket.\n");
      perror("QueryServer Error");

      QbCloseSocket(socketDescriptor);
      return(-1);
    }

    printf("Get request from client %s from port = %d\n", 
        inet_ntoa(sockAddr.sin_addr), portNumber);
    /* read command from socket */
    nbytes = recv(socketConnection, buffer, bufferLength, 0 ) ;
    /* query command read */
    if( 0 == strncmp(buffer,"EXITQBIC",8) ) break ; // exit issued
    if( ! IsDigits(buffer)) continue ;  // skip any request with text
        
    nbytes = send(socketConnection, okay, sizeof(okay) + 1, 0);
    sscanf(buffer,"%lu",&querySpecSize) ;
    if( querySpecSize <= 0 ) continue ; 
    
    query = QbCreateQueryClass("QbQueryServerClass") ;
    querySpec = new char[querySpecSize] ; 
    
    nbytes = QbRecvSocket(socketConnection, querySpec, querySpecSize ) ;
    
    results = query->Evaluate( querySpec ) ;
    if( results ) {
      results->ToQueryString( datum ) ; // convert the results to a string
      datum.Get( datumSize, datumValue )  ;
    } else datumSize = 0 ; 

    sprintf(buffer,"%lu %d",datumSize, query->GetReturnCode() ) ;

    nbytes = send(socketConnection, buffer, strlen(buffer) + 1, 0);

    delete query ;
    delete querySpec ; 

    if( datumSize ) { 
      nbytes = recv(socketConnection, buffer, bufferLength, 0 ) ; // get handshake
      nbytes = QbSendSocket(socketConnection, datumValue, datumSize);
    }
    
    recv(socketConnection, buffer, sizeof(nbytes), 0 ) ; 
    QbCloseSocket(socketConnection) ; 
  }
  QbCloseSocket(socketDescriptor);
  QbCleanSocket();

  
  return 0 ; 
}

int main( int argc, char **argv, char **envp )
{
  int portNumber = 0, rc ; 
  char *dsName = getenv("QbicDatabaseName") ;
  char *qbicServerPort = getenv("QbicServerPort") ; 

  if ( qbicServerPort ) portNumber = atoi( qbicServerPort ) ; 
  if( argc > 1 ) dsName = argv[1] ; 
  if( argc > 2 ) portNumber = atoi( argv[2] ) ;

  if( 0 == portNumber ) portNumber = 5517 ;
  if( NULL == dsName ) dsName = "QbicData" ; 
  
  rc = QbQueryServer( dsName, portNumber ) ;
  
  return rc ; 
}
