/*
 *  (C) COPYRIGHT International Business Machines Corp. 1995, 1996
 *
 * Filename : 73
 * Author   : M. Flickner
 * Date     : 8/5/96 11:01:44
 * Version  : 1.7.1.1
 * Purpose  : Qbic Generic Data Class
 *
 */

#ifndef __QbGenDat_hpp
#define __QbGenDat_hpp
#include <dmbsystm.h>

//
// Define an abstract type for generic data.  This allows this
// framework to be used with images, speech, text, graphics....
// 
class QbGenericDataClass: public QbBaseClass {

public:
    enum dataSource { unknown='u', memory='m', sample='s',
                      example='e', picker='p' } ;

private:
  QbGenericDataClass( const QbGenericDataClass& ) ; // avoid copies
  QbGenericDataClass& operator =(const QbGenericDataClass&) ;

protected:
  dataSource type ;
  QbDatumClass derivedClassData ;
  QbDatumClass exampleOrSampleFromDatabase ; 
  char *catalogName ; 
  char *keyName ; 
  char *samplerName ;
  Boolean isQuery ;      // True if this is the query specification
  char *textDescription ; 
  
public:
  DMBEXPHCONSTRUCT QbGenericDataClass();
  
  virtual ~QbGenericDataClass() {
    delete keyName ;
    delete catalogName ;
    delete samplerName ;
    delete textDescription ; 
  } ; 
  
  // converts a generic image description into an ascii string 
  virtual int ToQueryString(QbDatumClass &datum) ;
  
  // convert ascii query string filling in appropriate fields
  virtual int FromQueryString(const QbDatumClass &datum,
                              const char *featureClassname ) ; 

  const QbDatumClass &GetDerivedClassData() const
  { return derivedClassData ; } 

  // This should be renamed ...
  const QbDatumClass &GetData() const
  { return exampleOrSampleFromDatabase ; } 
  
  void SetType( const dataSource typeArg ) { type = typeArg ; }
  const dataSource GetType( ) const { return type ; }

  const char * GetKey() const { return keyName ; } 
  void SetKey( const char *keyArg ) {
    delete keyName;
    keyName = QbStrDup( keyArg);
  }
  
  const char *GetCatalog() const { return catalogName ; }
  void SetCatalog( const char *catalogArg ) {
    delete catalogName ;
    catalogName = QbStrDup( catalogArg ) ; 
  }

  const char * GetSampler() const { return samplerName ; }
  void SetSampler( const char *samplerArg ) {
    delete samplerName ;
    samplerName = QbStrDup( samplerArg ) ; 
  }

  Boolean GetIsQuery() const { return isQuery ; }
  Boolean SetIsQuery(Boolean boolean) { return isQuery=boolean ; }

  void SetTextDescription( const char *text ) {
    delete textDescription ; 
    textDescription = QbStrDup( text );
  }
  char *GetTextDescription() const { return textDescription; }

//
// From QbBaseClass
//  
  virtual const QbInterfaceType IType() const
  { return GenericDataInterfaceType ; }
  
  virtual const char *Type() const { return "QbGenericDataClass" ; }


};

#endif 
