/*
 *  IBM Confidential
 *  (C) COPYRIGHT International Business Machines Corp. 1995
 *
 * Filename : 78
 * Author   : Denis Lee
 * Date     : 7/8/96 16:17:39
 * Version  : 1.4.1.1
 * Purpose  : High level client side QBIC function specification
 *            for query submission and evaluation
 *
 */

#ifndef __QbQueryC_hpp
#define __QbQueryC_hpp

#include "QbAbsCls.hpp"

class QbQueryClientClass : public QbQueryClass {
public:

//
// This is a OQS style query evaluator
//
// Query specifications should be created using the function
// QbBuildQueryString.  An outline of the format is below however,
// users should ALWAYS use QbBuildQueryString.
//
//  query resultNumber catalogName
//  [[\[ restrictionList \]] [& p ( \[ partOfRestrictionList \] )]]
//  {
//    [ [&]     \[ featureClassname parameters genericData \]
//    [ [&] p ( \[ featureClassname parameters genericData \]
//    [  &      \[ featureClassname parameters genericData \] ...] ) ] ]
//       ...
//  }
//
// restrictionList = QbKeyDatabaseClass.ToQueryString() result
// genericData = QbGenericDataClass.ToQueryString() result
// parameters = QbParameterClass.ToQueryString() result
//
// This result of the query is in the returned QbKeyDatabaseClass 
//
  virtual const QbKeyDatabaseClass * Evaluate(const char *querySpecification); 
  
  QbQueryClientClass() {} ;
  virtual ~QbQueryClientClass() {} ;
  
  // from QbBaseClass 
  virtual const QbInterfaceType IType() const
  { return QueryInterfaceType ; } 
  virtual const char *Type() const { return "QbQueryClientClass" ; }
}; 

//
// routine to read until all bytes needed are obtained
//
extern unsigned long QbSocketRead(int socket,
                                  unsigned char *buffer,
                                  unsigned long neededBytes) ;

#endif
