/*
 *  IBM Confidential
 *  (C) COPYRIGHT International Business Machines Corp. 1995
 *
 * Filename : 79
 * Author   : M. Flickner
 * Date     : 5/1/96 09:55:13
 * Version  : 1.7
 * Purpose  : QBIC server query evaluator class
 *          
 */

#ifndef __QbQueryS_hpp
#define __QbQueryS_hpp

#include <dmbsystm.h>
#include "QbAbsCls.hpp"
#include "QbCatCls.hpp"

class QbQueryServerClass : public QbQueryClass {

private:
  enum {maxNumberOfFeatures=16} ;
  unsigned long queryResultsRequested ; // how many items do we return
  int numberObjectFeatures ;
  int numberSceneFeatures ;
  char catalogName[256] ;
  QbCatalogClass *catalog ;
  const QbConnectClass *connect ;

  QbKeyDatabaseClass *sceneRestrictionList ;
  QbFeatureClass *sceneFeatures[maxNumberOfFeatures] ;
  QbGenericDataClass *sceneGenericData[maxNumberOfFeatures] ;
  QbParameterClass *sceneParameters[maxNumberOfFeatures] ;
  float sceneFilterCost[maxNumberOfFeatures] ;
  float sceneDistanceCost[maxNumberOfFeatures] ;
      
  QbKeyDatabaseClass *objectRestrictionList ;
  QbFeatureClass *objectFeatures[maxNumberOfFeatures] ;
  QbGenericDataClass *objectGenericData[maxNumberOfFeatures] ;
  QbParameterClass *objectParameters[maxNumberOfFeatures] ;
  float objectFilterCost[maxNumberOfFeatures] ;
  float objectDistanceCost[maxNumberOfFeatures] ;

  QbFeatureClass *evaluateDistanceFeature ; 

  int Create( const char *query ) ; // create the query
  int Prepare() ;  // prepare and/or optimize the query
  int Execute() ;  // execute/run the query
  int Cleanup() ;  // cleanup storage for possible reuse of class
//
// This is a OQS style query evaluator
//
// Query specifications can be created using the function QbBuildQueryString.
// The result of a query is a QbKeyDatabaseClass.
//
public:  
  virtual const QbKeyDatabaseClass * Evaluate(const char *querySpecification); 
  
  DMBEXPHCONSTRUCT QbQueryServerClass() ;
  virtual DMBEXPHCONSTRUCT ~QbQueryServerClass() ;
  
  // from QbBaseClass 
  virtual const QbInterfaceType IType() const
  { return QueryInterfaceType ; } 
  virtual const char *Type() const { return "QbQueryServerClass" ; }

  //
  // The function below is needed to make UDF distance evaluation more
  // efficient.  This function can only be call if a query is first
  // evaluated - only works with single feature queryies .... 
  // 
  virtual int EvaluateDistance( const char *key,
                                const float abortDistance,
                                float &distance ) ; 
}; 

//
// Utility function to make a query string
//
DMBEXPHP( int)
QbBuildQueryString( const int numberOfResults,
                    const char *catalogName,
                    const QbKeyDatabaseClass *restrictionList,
                    const QbKeyDatabaseClass *partOfRestrictionList,
                    const int numberOfFeatures,
                    const char **featureClassnames,
                    QbParameterClass **parms,
                    QbGenericDataClass **data,
                    QbDatumClass &queryString ) ;

#endif
