/*
 *  IBM Confidential
 *  (C) COPYRIGHT International Business Machines Corp. 1995
 *
 * Filename : 65
 * Author   : Myron Flickner
 * Date     : 7/9/96 16:38:03
 * Version  : 1.3.1.1
 * Purpose  : Simple string class
 *
 */

#ifndef __QbStringClass_hpp
#define __QbStringClass_hpp

#include <string.h>
#include <ctype.h>

class QbStringClass {

private:
  unsigned long allocatedSize ;
  unsigned long currentSize ; // size includes the '\0' 
  char *buffer ;
  const int destructorDelete ; 
  
  QbStringClass( const QbStringClass& ) ; // no copies

  void IncreaseSpace(unsigned long spaceWanted )
  {
    char *tmpString ; 
    if( allocatedSize >= spaceWanted ) return ;
    
    // double the allocatedSize or use the requested space
    if( 2*allocatedSize > spaceWanted )
      allocatedSize *= 2 ; 
    else allocatedSize = spaceWanted ; 
    
    tmpString = buffer ; // save the old string 
    buffer = new char[ allocatedSize ] ;
    if( currentSize ) 
      memcpy( buffer, tmpString, currentSize ) ; // copy over the data
    delete tmpString ; // delete the old buffer
    
    return ;
  }
  
public:
  
  // Simple constructor from a string
  
  QbStringClass( const char *string, int destructorDeleteArg = 1 ) :
    allocatedSize(0), currentSize(0),buffer(NULL),
    destructorDelete(destructorDeleteArg)  {
      if( string ) {
        currentSize = strlen( string ) + 1  ;
        allocatedSize = currentSize*2 ;
      } else {
        allocatedSize = 64 ;
        currentSize = 0 ;
      }
      buffer = new char[allocatedSize] ;
      buffer[0] = '\0' ; 
      if( string ) memcpy( buffer, string, currentSize );
  }

  // Constructor to specify the size of the buffer
  
  QbStringClass( const unsigned long size,
                 int destructorDeleteArg = 1 ) :
    allocatedSize(1), currentSize(0), buffer(NULL), // always allocate 1 byte
    destructorDelete(destructorDeleteArg)  {
      if( size ) allocatedSize = size ;
      buffer = new char[allocatedSize] ;
      buffer[0] = '\0' ;
  }
  
  // destructor
  
  ~QbStringClass() {
    if( destructorDelete ) delete buffer ;
  }

  // allow for a cast to a unsigned char and char 

  operator unsigned char * () const { return (unsigned char *) buffer ; }
  operator char * () const { return buffer ; }
  operator const char *() const { return buffer ; } 

  QbStringClass& operator = (const char *in) {
    int length ;
    length = strlen( in ) + 1 ;
    IncreaseSpace( length ) ;
    memcpy( buffer, in, length ) ;
    currentSize = length ; 
    return *this ; 
  }
  
  unsigned long GetSize() const { return currentSize ; }

  
  // concatenate to the end of the string
  
  QbStringClass& operator += ( const char *str )
  {
    unsigned long length, spaceNeeded ;
    
    length = strlen( str ) ;
    // handle case when currentSize = 0 ; 
    spaceNeeded = (currentSize ? currentSize : 1 ) + length ;
    IncreaseSpace(spaceNeeded) ; 

    if( 0 == currentSize )
      memcpy( &buffer[currentSize], str, length + 1 ) ;
    else
      memcpy( &buffer[currentSize-1], str, length + 1 ) ;
    
    currentSize = spaceNeeded ; 
    return *this ; 
  }
  
  //
  // Encode a binary buffer and concat it to the string. For now we hex
  // encode - evenutally we need to mime encode and possible lzw compress
  // and mime encode
  //
  int EncodeAndConcat( const unsigned char *input, const unsigned long size)
  {
    unsigned long i,j, spaceNeeded ;
    char *output ;
    static const char hexTable[] = "0123456789abcdef" ;
    
    spaceNeeded = (currentSize ? currentSize : 1 ) + 2*size ; 
    IncreaseSpace( spaceNeeded ) ;
    if( 0 == currentSize ) output = &buffer[currentSize] ;
    else output = &buffer[currentSize-1] ;
    
    for(j=i=0; i<size; i++) {
      output[j++] = hexTable[ input[i] >> 4 ] ;
      output[j++] = hexTable[ input[i] & 0x0f ] ; 
    }
    output[j] = '\0' ; // null terminate
    
    currentSize = spaceNeeded ;
    return 0 ; 
  }
  
} ;

static inline int hexToInt( const char c )
{
  switch ( tolower(c) ) {
  case '0' : return 0 ;
  case '1' : return 1 ;
  case '2' : return 2 ;
  case '3' : return 3 ;
  case '4' : return 4 ;
  case '5' : return 5 ;
  case '6' : return 6 ;
  case '7' : return 7 ;
  case '8' : return 8 ;
  case '9' : return 9 ;
  case 'a' : return 10 ;
  case 'b' : return 11 ;
  case 'c' : return 12 ;
  case 'd' : return 13 ;
  case 'e' : return 14 ;
  case 'f' : return 15 ;
  default: return 0 ;
  }
}

// function to reverse EncodeAndConcat 
static inline int DecodeBuffer( const char *encoded, unsigned char *decode,
                                unsigned long decodeSpace )
{
  unsigned long i,j;
  for(j=i=0; isxdigit( encoded[i] ); i+=2 ) {
    decode[j++] = (hexToInt(encoded[i]) << 4 ) | hexToInt( encoded[i+1] ) ;
    if( j >= decodeSpace ) return -1 ; 
  }
  return 0 ; 
}
#endif
