/*
 *                     (C) COPYRIGHT IBM CORP 1991
 *
 * Filename : 76
 * Author   : M. Flickner (and friends)
 * Date     : 8/15/96 13:31:55
 * Version  : 1.18
 * Purpose  :
 *
 */
/***************************************************************************/
/***  COMPONENT_NAME:    ULTIMEDIA COMMON INCLUDE                        ***/
/***                                                                     ***/
/***  IBM CONFIDENTIAL -- (IBM Confidential Restricted when              ***/
/***  combined with the aggregated modules for this product)             ***/
/***  OBJECT CODE ONLY SOURCE MATERIALS                                  ***/
/***                                                                     ***/
/***  (C) COPYRIGHT International Business Machines Corp. 1993, 1994     ***/
/***                Personal Software Products(PSP)                      ***/
/***  All Rights Reserved                                                ***/
/***  US Government Users Restricted Rights - Use, duplication or        ***/
/***  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  ***/
/***************************************************************************/
#ifndef __Qbic_h
#define __Qbic_h

#include <stdio.h>

#ifndef NO_X11
//
// This is a hack to avoid the name conflict with 'vector' in Image Assistant
// and Numeric Recipes in C.  This should be cleaned up someday, somehow.
// COLOR, SCREEN & SAME have name collisions with xvt so we hack around it. 
//
  #define vector ImpVector
  #define COLOR ImpCOLOR
  #define SCREEN ImpSCREEN
  #define SAME ImpSAME
  #include "locdefs.h" /* this includes impdat,impdef, ... */
  #undef vector
  #undef SAME
  #undef COLOR
  #undef SCREEN
  #include "impptype.h"
 
  #include <X11/Intrinsic.h>
#else
//
// To compile the QBIC API without impart or X11 we need the
// following defines/includes
//
  #include <string.h>
  #include "QbImpRtn.h"
  typedef int Bool ; 
  typedef void * Widget ; // bogus definition of the unused X11 types 
  typedef void * Pixmap ;
  typedef void * Position ;
  typedef void * XEvent ;
  typedef void * XArc ;
  typedef void * Pixel ; 
  typedef struct {
    short x, y;
  } XPoint;
#if !defined(_ISYNONYM_)  
  typedef enum { False = 0, True = 1 } Boolean;
#else
#if defined(DMB_AIX)
typedef int Boolean ;
#endif
#endif

#endif

#include <math.h>

#include <stdlib.h>
#include "quError.h"

#if !defined(M_PI)
#define M_PI       3.14159265358979323846264338327950288
#endif

typedef double fastfloat;

#define COLOR_DIMENSIONS 3

#define COLOR_DISTANCE_SIGMA 7.0

#define COLOR_MATCH_STRING "m13"

#define USE64COLORS 0   // set to 1 for old Mountain View code

/* new stuff for adjusting the MTM metric */
typedef enum { MTMEUC, MTMQFM, MTMHUE } QbMTMmetricType ;
//#define MTMEUCnormalizer 0.000001
//#define MTMQFMnormalizer 0.000001
//#define MTMHUEnormalizer 0.000001538
/* these versions normalize to less than 1.0 and assume the MTM
   coordinates are in their natural range (not in FIXED100 form) */
#define MTMEUCnormalizer 3.76446e-4     
#define MTMQFMnormalizer 2.83421e-4
#define MTMHUEnormalizer 3.99469e-4

// JLH: (10/6/95) 
// replaced all instances of COLOR_HISTOGRAM_BUCKETS by ORIG_HISTOGRAM_BUCKETS
// replaced all instances of OldHistogramColors by OrigHistogramColors
// replaced all instances of RGBHistogramColorIndices by OrigRGB...
#if USE64COLORS
#define ORIG_HISTOGRAM_BUCKETS 64 
#define OrigRGBHistogramColorIndices OrigRGBHistogramColorIndices64
#define OrigHistogramColors OrigHistogramColors64
#else
#define ORIG_HISTOGRAM_BUCKETS 256
#define OrigRGBHistogramColorIndices OrigRGBHistogramColorIndices256
#define OrigHistogramColors OrigHistogramColors256
#endif

#define DRAW_HISTOGRAM_BUCKETS 64    /*  <= ORIG_HISTOGRAM_BUCKETS */
#define DrawRGBHistogramColorIndices OrigRGBHistogramColorIndices64
#define DrawHistogramColors OrigHistogramColors64

// Aditya: Local Histogram Buckets are either 64 or 256 in size
#define LOCAL_HISTOGRAM_SIZE64 1
#if LOCAL_HISTOGRAM_SIZE64
#define LOCAL_HISTOGRAM_BUCKETS DRAW_HISTOGRAM_BUCKETS
#define LocalHistogramSimilarityMatrix DrawColorSimilarityMatrix
#define LocalHistogramColorIndices DrawRGBHistogramColorIndices
#define LocalHistogramColors DrawHistogramColors
#else
#define LOCAL_HISTOGRAM_BUCKETS ORIG_HISTOGRAM_BUCKETS
#define LocalHistogramSimilarityMatrix ColorSimilarityMatrix
#define LocalHistogramColorIndices OrigRGBHistogramColorIndices
#define LocalHistogramColors OrigHistogramColors
#endif

#define  MAX_DB_LINELENGTH 65536    /* max size of line in database */
/************************************************************************
  As of 8/2/93, MAX_DB_LINELENGTH needs to exceed the size of positional
  color data which is
     NUM_PARTITIONS*( TOP_N_COLORS*2*[%5d+blank] + 3*[%5d+blank] ) + 30,
  where 30=space reserved for imagename and skipflag
*************************************************************************
  As of 3/31/94, MAX_DB_LINELENGTH needs to exceed the size of draw feature
  data which is
     MAXDRAWGRIDHxV*( 3*[%5+blank] + DRAW_TOP_N_COLORS*2*[%5d+blank] +
             3*[%5d+blank] ) + 30 + 2*[%2+blank] + 
	     DRAW_HISTOGRAM_BUCKETS*[%4+blank]
  where
  -- first 3*x is for average color data,
  -- next chunk is for top histogram colors,
  -- next 3*x is for texture info (if used)
  -- 30=space reserved for imagename and skipflag
  -- 2*y is for grid size of data.
  -- last chunk is for the total scene histogram
************************************************************************/
/* big number initializing distance for invalid or featureless query */
#define NO_FEATURE_VALUE 1000000.0

typedef unsigned long QbSkipType;/* Indicate which features are unavailable */
#define QbSkipNone             0x0000
#define QbSkipQuery            0x0400

#define QbSkipColorRGB         0x0001
#define QbSkipColorLAB         0x0002
#define QbSkipColorYIQ         0x0004
#define QbSkipColorMTM         0x0008
#define QbSkipColorHIST        0x0010

#define QbSkipTextureTam       0x0001

#define QbSkipLocationAll       0x0001

#define QbSkipShapeHuMoments    0x0001
#define QbSkipShapeGenEig       0x0002
#define QbSkipShapeTurningAngle 0x0004
#define QbSkipShapeAmi          0x0008
#define QbSkipShapeCurveSign    0x0010
#define QbSkipShapeCp           0x0020
#define QbSkipShapeGaussScaleSpace  0x0040
#define QbSkipShapeEllpGaussScaleSpace 0x0080
/*
 * mask for any of the spline based features
 */
#define QbSkipShapeSpline   (QbSkipShapeCp|QbSkipShapeTurningAngle|QbSkipShapeCurveSign)

#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#endif
#ifndef SGN
#define SGN(x)         ( (x) < 0 ? (-1) : (1) )
#endif
#ifndef ABS
#define ABS(a)         ( (a) < 0 ? -(a) : (a) )
#endif
#ifndef SQR
#define SQR(a)         ( (a) * (a) )
#endif
#ifndef FF
#define FF(x) ((fastfloat) (x))
#endif


#define QBCOBJ(scene)   (scene)->objects[(scene)->CurrentObj]
#define QBCOBJID(scene) ((scene)->CurrentObj==-1 ? -1 : (scene)->objects[(scene)->CurrentObj].id)
#define QBOBJ(scene, i) (scene)->objects[(i)]

#define IMP_ERR_CHECK  if (rc != IMPOK) { impomt(&rc); exit(-1); }
typedef enum { MaskEnum, SketchEnum } QbMaskEnumType ;

typedef enum {
#ifndef NO_X11
// avoid these for API version of qbic to avoid name conflict with stl symbols
               BITMAP, BOUNDRY, LINE,  RECTANGLE,
#endif  
               ELLIPSE, PAINT, ERASE, DRAG, ROTATE,
               FILL, SBOUNDRY, SEGMENT } OutlineEnumType ;


#define RADIAN_TO_DEGREE(x) ((x)*(180.0/M_PI))
#define DEGREE_TO_RADIAN(x) ((x)*(M_PI/180.0))

#define STRING_MALLOC( str ) QbMalloc( strlen(str)+1,QERR_MEM,QERR_FATAL )

#define DYNAMIC_STRING_COPY( dest, src ) \
  strcpy( dest = (char *) STRING_MALLOC( src ), src )

#define DEFAULT_NUMBER_OBJECTS 4


#define LASTPIXELIMP1(image)  ( (image).datptr + (image).numpels - 1)

#define QBPX8( imageptr,x,y ) ((imageptr)->datptr[ (x) + \
                                                  (y)*((imageptr)->xsize) ])

#define QBGetPelWithCheck( image,x,y )  (( (x) < 0 || (y) < 0 || \
  (x) >= ((image)->xsize) || (y) >= (image)->ysize ) ? \
  0 : (image)->datptr[ (x) + (y)*((image)->xsize) ])

#ifndef ROUND
#define ROUND(x)  ( ( (x) >= 0 ) ? (x) + 0.5 : (x) - 0.5 )
#endif
/*
 *  Fixed100 represents a number scaled by 100;
 */
typedef long Fixed100;
#define FIXED100_TO(x,type)  ( (type) ( (fastfloat) (x) / 100.0 ) )
#define FIXED100(x)  ( (Fixed100) \
                      (x >= LONG_MAX*0.01) ? LONG_MAX :\
                      ((x <= LONG_MIN*.01) ? LONG_MIN:\
                      ROUND( (x) * 100.0) ) )

/* structure for holding results of a query */
typedef struct
{
  unsigned long idx;   /* index of feature in test array */
  float         dist;  /* distance of feature from reference */
} QbDistResultType;

#define LUTSIZE 256
#define IMAGE_GAMMA 2.22
#define CRT_GAMMA 2.22

/* declares for color position matching */
#define  TOP_N_COLORS  8
#define  X_PARTITIONS  16
#define  Y_PARTITIONS  16
#define  NUM_PARTITIONS  X_PARTITIONS * Y_PARTITIONS

typedef struct {
  QbSkipType SkipFlag;               /* Indicate which features are not available */
  Fixed100 ColorAvgsRGB[COLOR_DIMENSIONS];          /* average RGB value */
  Fixed100 ColorAvgsLab[COLOR_DIMENSIONS];          /* average Lab value */
  Fixed100 ColorAvgsYiq[COLOR_DIMENSIONS];          /* average Yiq value */
  Fixed100 ColorAvgsMTM[COLOR_DIMENSIONS];          /* average MTM value */
  short RGBHist[ORIG_HISTOGRAM_BUCKETS];  /* color histogram */
} QbColorFeatureType ;

/* New Feature type, Color Histogram of 5 blocks in an image */
#define NUM_BLOCKS 5
typedef struct {
/*  QbSkipType SkipFlag;               Indicate which features are not available */
  short RGBHist[NUM_BLOCKS*LOCAL_HISTOGRAM_BUCKETS];  /* color histogram */
  int Mean[NUM_BLOCKS][3];  /* MTM means for the blocks */
} QbLocalColorHistogramFeatureType ;

typedef struct {
  unsigned char  index;       /* index into the color histogram */
  unsigned char  count;       /* count at this value */
} QbTopNColorsType;

typedef struct {
  Fixed100  ColorAvgsMTM[COLOR_DIMENSIONS];  /* average MTM value */
  QbTopNColorsType  TopNColors[TOP_N_COLORS];  /* color and count of top N colors */
} QbColorPositionElementType ;

typedef struct {
  QbSkipType SkipFlag;        /* Indicate which features are not available */
  QbColorPositionElementType Element[NUM_PARTITIONS];
} QbColorPositionFeatureType ;

/* declares for draw color matching */
#define MAXDRAWGRIDH 24
#define MAXDRAWGRIDV 18
#define MAXDRAWGRIDHxV  MAXDRAWGRIDH*MAXDRAWGRIDV
#define  DRAW_TOP_N_COLORS 5   /* try to get less data for efficiency */
#define DRAW_TEXTURE_DIMENSIONS 3
#define DRAW_INDEX_FEATURES 
//  we disable these for now.  The first doesn't seem to be too
//  useful and the second hasn't really been tested.
/* #define DRAW_HISTOGRAM_FEATURE 1 */
/* #define DRAW_TEXTURE_FEATURE 1 */
typedef struct {
  Fixed100          ColorAvgsMTM[COLOR_DIMENSIONS]; /* average MTM value */
#ifdef DRAW_INDEX_FEATURES
  unsigned char     ColorAvgsMTMDrawIndex ;
  unsigned char     ColorAvgsMTMHistIndex ;
  unsigned char     TopColorHistIndex ;
#endif
#ifdef DRAW_HISTOGRAM_FEATURE
  QbTopNColorsType  TopNColors[DRAW_TOP_N_COLORS];  /* color and count of top N colors */
#endif 
#ifdef DRAW_TEXTURE_FEATURE
  Fixed100        Texture[DRAW_TEXTURE_DIMENSIONS] ;   /* add texture later? */
#endif
} QbDrawElementType ;

typedef struct {
  short gridh ;    /* number of horizontal divisions */
  short gridv ;    /* number of vertical divisions */
} QbDrawGridType ;

/* added these structures so pickerFeatures carry it's object structure
   with it (more reentrant). 9/14/95 (JLH) */
typedef struct {
  short DrawHist[DRAW_HISTOGRAM_BUCKETS] ; /* histogram for filtering */
  short DrawHistMass ;                     /* mass of histogram <=1000 */
} QbDrawHistDataType ;

typedef struct {
  short i ;  
  short j ;
  short bdry ;                  /* non-zero if boundary element */  
  double **rawDistTable ;   /* element to element matching data */
  double **burstDistTable ; /* if needed, element to burst match data */
  double **haloDistTable ;  /* if needed, element to halo match data */  
  double **matchDistTable ; /* ptr to burst data if burst match, else raw */
} QbObjectIndicesType ;

typedef struct {
  short maxU ;    /* max offset up ( negative Y ) */
  short maxD ;    /* max offset down ( positive Y ) */
  short maxL ;    /* max offset left ( negative X ) */
  short maxR ;    /* max offset right ( positive X ) */
} QbObjectOffsetType ;

typedef struct {
  short counter ; /* number of items in object */
  short bdrycntr ; /* number of items in (interior) boundary of object */
  QbObjectIndicesType *indices ;  /* array of [i][j] pairs for object */
  QbObjectOffsetType offsets ;  
} QbObjectListType  ;


typedef struct {
  QbObjectListType *ObjectList ;
  short ObjectLabels[MAXDRAWGRIDV][MAXDRAWGRIDH] ;
  short numObjects ;
} QbDrawObjectDataType ;

/* changed this data structure to carry the picker matching info with it */
typedef struct {
  QbSkipType SkipFlag;        /* Indicate which features are not available */
  QbDrawElementType Element[MAXDRAWGRIDV][MAXDRAWGRIDH];/* oversize 2d array */
  QbDrawGridType DrawGridSize;   /* encodes the size of Element used */
  QbDrawHistDataType DrawHistData ;  /* histogram struct used for filtering */
  QbDrawObjectDataType *DrawObjectData ; /* a place to keep query object
                                            structure with the feature data;
                                            NULL for database data */
} QbDrawFeatureType ;

/* Structures for color region features follow */
typedef struct regionTreeNode {
  struct regionTreeNode *parent;
  struct regionTreeNode *firstChild;
  struct regionTreeNode *nextSibling;
  struct regionTreeNode *matchingNode;
  fastfloat matchingDistance;
  fastfloat matchingLowerBound;
  int numLeafs;
  fastfloat rectangle[6];  /* (corner[2], unit[2], width, height) */
} regionTreeNodeType ;

typedef struct QbColorRegionFeature {
  QbSkipType SkipFlag;        /* Indicate which features are not available */
  int xSize;
  int ySize;
  int numColors;
  int numColorsAllocated; /* Amount of memory pointed to by colorVector */
  int numNodes;     /* Total number of regionTreeNodes in the trees */
  int numNodesAllocated; /* number of nodes allocated in treeNodeMemory */
  regionTreeNodeType *treeNodeMemory; /* pointer to block of node memory */
  int *colorVector; /* in RGBRGB..RGB order */
  regionTreeNodeType **treeVector;
} QbColorRegionFeatureType ;

/* texture feature structure */

typedef struct  {
  QbSkipType SkipFlag;        /* Indicate which features are not available */
  Fixed100 TamCoarseness;          /* Tamura coarseness */
  Fixed100 TamContrast;            /* Tamura contrast */
  Fixed100 TamDirectionality;      /* Tamura directionality */
} QbTextureFeatureType ;

/* text feature structure */

typedef struct _QbTextFeatureType {
    QbSkipType SkipFlag;      /* Indicate which features are not available */
    char *text;
    int size;
} QbTextFeatureType;

/* location feature structure (all normalized) */

typedef struct  {
  QbSkipType SkipFlag;        /* Indicate which features are not available */
  Fixed100 AreaCentroidX, AreaCentroidY; /* centroid of area */
  Fixed100 Area;                         /* normalized area */
  Fixed100 MinX, MaxX, MinY, MaxY;       /* bounding box */
} QbLocationFeatureType ;

/*
 * Structure for active query specifications
 */

typedef struct
{
  fastfloat locX;
  fastfloat locY;
} QuerySpecType;
extern QuerySpecType CurQuerySpec;

typedef struct {
  int prescore_array_size;
  int prescore_array[128];
} QbSketchPSFeatureType;

typedef struct {
  short image[4096];                /* 64x64 reduced image */
} QbSketchImFeatureType;

#define MAX_NUMBER_AMI_FEATURES 64
#define MAX_NUMBER_GENEIG_FEATURES 64
#define MAX_NUMBER_SPLINE_FEATURES 2048
#define MAX_GAUSSIANS 8 /* how many resolutions for multi-scale */
#define MAX_ELLP_GAUSSIANS 1 /* how many scales for ellitpical gaussians */
#define MAX_ANGLES 8 /* how many orientations for elliptical gaussians */
#define MAX_NUMBER_CP_FEATURES 4096
#define MAX_NUMBER_CURV_FEATURES 4096

typedef struct  {
  QbSkipType SkipFlag; /* Indicate which features are not available */
  unsigned long area ;                    /* blob area        */
  Fixed100 xCentroid, yCentroid ;         /* x and y centroid */
  Fixed100 MajorAxisOrientation ;         /* Orientation of Major Axis */
  Fixed100 Eccentricity;                  /* Ratio of the eigenvalues  */
  Fixed100 peround ;                      /* perimeter^2/area */
  /*
   * Space for the Algebraic Moment Invariants.  For details see
   * Recognition and Positioning of Rigid Object Using Algebraic
   * Moment Invariants.  Gabrial Taubin.  SPIE Vol 1570, Geometric
   * Methods in Computer Vision, page 175, San Diego, 1991.
   */
  int NumberAmi ; /* actual number of features used */
  Fixed100 ami[MAX_NUMBER_AMI_FEATURES] ;

  int NumberGenEig ; /* actual number of features used */
  Fixed100 geneig[MAX_NUMBER_GENEIG_FEATURES] ;

  int NumberResolutions ;
  int ControlPointsPerResolution[16] ; /* Number control points at each resolution */
  XPoint ControlPoints[MAX_NUMBER_CP_FEATURES] ; /* cp data */

  int NumberCurvatureResolutions ;
  int CurvaturePointsPerResolution[16] ; /* Number of curvature sample points
                                            at each resolution */
  unsigned long CurvaturePoints[MAX_NUMBER_CURV_FEATURES] ;

  int NumberSpline ;    /* total number of features used */
  int NumberSamples;   /* number of samples per resolution */
  int NumberTALists;
  Fixed100 spline[4][MAX_NUMBER_SPLINE_FEATURES];

  int NumberGaussians ;   /* Number of Guassian used for multi-scale */
  long GaussSigma[MAX_GAUSSIANS]; /* Gaussian standard deviation*/
  long Gauss[MAX_GAUSSIANS] ;     /* Gaussian value at centroid */
  long GaussXd1[MAX_GAUSSIANS] ;  /* Gaussian x derivative at centroid */
  long GaussYd1[MAX_GAUSSIANS] ;  /* Gaussian y derivative at centroid */
  long GaussXd2[MAX_GAUSSIANS] ;  /* Gaussian x^2 derivative at centroid */
  long GaussYd2[MAX_GAUSSIANS] ;  /* Gaussian y^2 derivative at centroid */
  long GaussXYd2[MAX_GAUSSIANS] ;  /* Gaussian xy derivative at centroid */


  /* The following list is for elliptical gaussians */
  int NumberEllpGaussians;   /* Number of elliptical gaussian scales */
  long EllpGaussSigma1[MAX_ELLP_GAUSSIANS]; /* Gaussian sigma - major axis */
  long EllpGaussSigma2[MAX_ELLP_GAUSSIANS]; /* Gaussian sigma - minor axis */
  long EllpGauss[MAX_ELLP_GAUSSIANS][MAX_ANGLES]; /* Gaussian value at centroid - one for each scale and one for each angle */
  long EllpGaussXd1[MAX_ELLP_GAUSSIANS][MAX_ANGLES]; /* Gaussian x derivative at centroid */
  long EllpGaussYd1[MAX_ELLP_GAUSSIANS][MAX_ANGLES]; /* Gaussian y derivative at centroid */
  long EllpGaussXd2[MAX_ELLP_GAUSSIANS][MAX_ANGLES]; /* Gaussian x 2nd derivative at centroid */
  long EllpGaussYd2[MAX_ELLP_GAUSSIANS][MAX_ANGLES]; /* Gaussian y 2nd derivative at centroid */
  long EllpGaussXYd1[MAX_ELLP_GAUSSIANS][MAX_ANGLES]; /* Gaussian xy derivative at centroid */

  /*p
   * The feature below are computed from the moments of the binary mask.
   * They are invariant to translation, rotation, scale.  All but
   * Phi7 are also invariant to reflection.
   * For details see page 380 in Fundamentails of Digital Image Processing
   * by Anil Jain ( equation for phi7 has a typeo in the book )
   */
  Fixed100 Phi1, Phi2 ;           /* Second order moment invariants     */
  Fixed100 Phi3, Phi4, Phi5, Phi6 ;  /* Third order moment invariants      */
  Fixed100 Phi7 ;                    /* Phi7 Changes sign under reflection */
} QbShapeFeatureType ;


/*****************MULTIRESOLUTION CODE***********/
typedef struct {
  short TrancatedValues[120*3];
  int DataPoints[120*6];
  fastfloat Average_Y;
  fastfloat Average_I;
  fastfloat Average_Q;
  fastfloat  RealValues[120*3];
} QbMultiresFeatureType;
/*************MULTIRESOLUTION CODE***************/
  
/*************FILTER CODE****************/
typedef struct {
  fastfloat CoefficientsY[405];
  fastfloat CoefficientsI[405];
  fastfloat CoefficientsQ[405];
  short CoordinatesY[2430];
  short CoordinatesI[2430];
  short CoordinatesQ[2430];
  fastfloat LargestY[120];
  fastfloat LargestI[120];
  fastfloat LargestQ[120];
  short LargestCoordY[120];
  short LargestCoordI[120];
  short LargestCoordQ[120];
} QbFilterFeatureType;
/***********FILTER CODE*****************/

/*************BOX FILTER CODE****************/
typedef struct {
  long boxCoefficientsRed[200];
  long boxCoefficientsGreen[200];
  long boxCoefficientsBlue[200];
} QbBoxFilterFeatureType;
/***********FILTER CODE*****************/

/*
 * Data structure definition for an (sub)object ;
 */
typedef struct QbObjectStruct {
  char *MaskFilename;
  char *text ;
  long id ;
  unsigned short xmax, ymax, xmin, ymin ; /* bounding box info for bitmap */
  Impvaru *bitmap ;
  Boolean valid ;
  Boolean changed ;
  Boolean selected ;
  QbColorFeatureType ColorFeatures ;
/***********LocalColorHistogram Code ********/
  QbLocalColorHistogramFeatureType localColorHistogramFeatures;
/***********Local Color Histogram Code ********/
  QbTextureFeatureType TextureFeatures ;
  QbShapeFeatureType ShapeFeatures ;
  QbLocationFeatureType LocationFeatures ;
/************MULTIRESOLUTION CODE*********/
  QbMultiresFeatureType multiresFeatures;
  QbFilterFeatureType filterFeature;
  QbBoxFilterFeatureType boxFilterFeature;
/***********MULTIRESOLUTION CODE*********/

  int coordindex ; /* Used for boundary descriptions */
  int coordend ;
  XPoint *CoordList ;
  /*
   * Fields below are X-window related
   */

  OutlineEnumType OutlineMethod ;
  Pixel color ;

  Pixmap pixmap, NormPixmap; /* Used for bitmap descriptions */

  XArc Arcparms ; /* Used for arc descriptions */

  Widget ListLineFrame ;
  Widget ListLineForm ;
  Widget ListLineSelectButton ;
  Widget ListLineText ;
  Widget ListLineButton ;
  QbMaskEnumType MaskType ;
} QbObjectType ;

typedef struct {
  QbSkipType SkipFlag;                    /* Indicate which features are not available */
  struct QbObjectStruct *parent ;
  Fixed100 NormX, NormY, NormArea ; /* Normalized centroid and area info */
} QbLayoutFeatureType ;

/*
 * Data structure definition for the scene
 */
typedef struct {
  char *filename ;
  char *SceneText ;
  short MaxNumberObjects ;
  short CurrentObj ;
  short LastDefinedObj ;
  QbObjectType *objects ;
  Impvaru *QbImage ;
  Impvaru *QbLut ;
  Impvaru *QbZoom1Image ;
  Impvaru *QbZoom2Image ;
  Impvaru *QbZoom4Image ;
  Impvaru *QbZoomHalfImage ;
  Impvaru *QbZoomQuarterImage ;
  float  ZoomFactor ;
  Bool updated ;
  Impvaru *QbDisplayImage ;         /* Repalletized image */
  Impvaru *QbDisplayLut ;           /* Lut for repalletization */
  Impvaru *QbActiveDisplayedImage ; /* Image that appears on screen */
  Impvaru *QbActiveDisplayedLut ;   /* Lut used on the screen */
  Widget QbImageWidget ;
} QbSceneType ;

/* for clustering routines */
typedef struct {
  double centroid[3];
  double weight;
  double tot_sq_error;
  char   pinned_flag;
} QbClusterType;

/*****************TIRE CODE*********************/
typedef struct {
  int xsize;
  int ysize;
  char *image;
} QbTireFeatureType;
/*****************TIRE CODE*********************/


#endif /* #ifndef __Qbic_h */
